@@ -1,3 +1,5 @@
+import sys
+
 import pandas as pd
 import pytest
 from dask.delayed import delayed
@@ -44,21 +46,24 @@ def test_dask_graph_adapter_simple(client):
     # TODO: do some more asserting?
 
 
+@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7")
 def test_smoke_screen_module(client):
     config = {"region": "US"}
-    index = pd.Series(pd.DatetimeIndex(pd.date_range(start="20200101", end="20220801", freq="7d")))
     dr = driver.Driver(config, smoke_screen_module, adapter=h_dask.DaskGraphAdapter(client))
     output_columns = [
         "raw_acquisition_cost",
         "pessimistic_net_acquisition_cost",
         "neutral_net_acquisition_cost",
         "optimistic_net_acquisition_cost",
+        "series_with_start_date_end_date",
     ]
     df = dr.execute(
-        inputs={"start_date": "20200101", "end_date": "20220801"}, final_vars=output_columns
+        inputs={"date_range": {"start_date": "20200101", "end_date": "20220801"}},
+        final_vars=output_columns,
     )
     epsilon = 0.00001
     assert abs(df.mean()["raw_acquisition_cost"] - 0.393808) < epsilon
     assert abs(df.mean()["pessimistic_net_acquisition_cost"] - 0.420769) < epsilon
     assert abs(df.mean()["neutral_net_acquisition_cost"] - 0.405582) < epsilon
     assert abs(df.mean()["optimistic_net_acquisition_cost"] - 0.399363) < epsilon
+    assert df["series_with_start_date_end_date"].iloc[0] == "date_20200101_date_20220801"
