@@ -13,7 +13,10 @@ import pandas as pd
 import typing_inspect
 from pandas.core.indexes import extension as pd_extension
 
-from . import node
+try:
+    from . import node
+except ImportError:
+    import node
 
 logger = logging.getLogger(__name__)
 
@@ -118,33 +121,32 @@ class PandasDataFrameResult(ResultMixin):
         # if there is more than one time index
         if time_indexes_length > 1:
             logger.warning(
-                f"WARNING: Time/Categorical index type mismatches detected - check output to ensure Pandas "
-                f"is doing what you intend to do. Else change the index types to match. Set logger to debug "
-                f"to see index types."
+                "WARNING: Time/Categorical index type mismatches detected - check output to ensure Pandas "
+                "is doing what you intend to do. Else change the index types to match. Set logger to debug "
+                "to see index types."
             )
             types_match = False
         # if there is more than one index type and it's not explained by the time indexes then
         if number_with_indexes > 1 and all_indexes_length > time_indexes_length:
             logger.warning(
-                f"WARNING: Multiple index types detected - check output to ensure Pandas is "
-                f"doing what you intend to do. Else change the index types to match. Set logger to debug to "
-                f"see index types."
+                "WARNING: Multiple index types detected - check output to ensure Pandas is "
+                "doing what you intend to do. Else change the index types to match. Set logger to debug to "
+                "see index types."
             )
             types_match = False
         elif number_with_indexes == 1 and no_index_length > 0:
             logger.warning(
-                f"WARNING: a single pandas index was found, but there are also {no_index_length} outputs without "
-                f"an index. Those values will be made constants throughout the values of the index."
+                f"WARNING: a single pandas index was found, but there are also {len(no_indexes['no-index'])} "
+                "outputs without an index. Please check whether the dataframe created matches what what you "
+                "expect to happen."
             )
             # Strictly speaking the index types match -- there is only one -- so setting to True.
             types_match = True
         # if all indexes matches no indexes
         elif no_index_length == all_indexes_length:
             logger.warning(
-                "It appears no Pandas index type was detected. This will likely break when trying to "
-                "create a DataFrame. E.g. are you requesting all scalar values? Use a different result "
-                "builder or return at least one Pandas object with an index. "
-                "Ignore this warning if you're using DASK for now."
+                "It appears no Pandas index type was detected (ignore this warning if you're using DASK for now.) "
+                "Please check whether the dataframe created matches what what you expect to happen."
             )
             types_match = False
         if logger.isEnabledFor(logging.DEBUG):
@@ -166,9 +168,12 @@ class PandasDataFrameResult(ResultMixin):
             (value,) = outputs.values()  # this works because it's length 1.
             if isinstance(value, pd.DataFrame):
                 return value
-            elif isinstance(value, pd.Series):
-                return pd.DataFrame(outputs)
-            raise ValueError(f"Cannot build result. Cannot handle type {value}.")
+
+        if not any(pd.api.types.is_list_like(value) for value in outputs.values()):
+            # If we're dealing with all values that don't have any "index" that could be created
+            # (i.e. scalars, objects) coerce the output to a single-row, multi-column dataframe.
+            return pd.DataFrame([outputs])
+
         return pd.DataFrame(outputs)
 
 
