@@ -10,6 +10,7 @@ from dask.delayed import Delayed, delayed
 from dask.distributed import Client as DaskClient
 
 from hamilton import base, node
+from hamilton.base import SimplePythonGraphAdapter
 
 logger = logging.getLogger(__name__)
 
@@ -78,7 +79,7 @@ class DaskGraphAdapter(base.HamiltonGraphAdapter):
             return True
         elif node_type == np.array and isinstance(input_value, dask.array.Array):
             return True
-        return node_type == typing.Any or isinstance(input_value, node_type)
+        return SimplePythonGraphAdapter.check_input_type(node_type, input_value)
 
     @staticmethod
     def check_node_type_equivalence(node_type: typing.Type, input_type: typing.Type) -> bool:
