@@ -1,5 +1,4 @@
 import functools
-import inspect
 import logging
 import typing
 
@@ -7,6 +6,7 @@ import ray
 from ray import workflow
 
 from hamilton import base, node
+from hamilton.base import SimplePythonGraphAdapter
 
 logger = logging.getLogger(__name__)
 
@@ -64,7 +64,7 @@ class RayGraphAdapter(base.HamiltonGraphAdapter, base.ResultMixin):
         # NOTE: the type of a raylet is unknown until they are computed
         if isinstance(input_value, ray._raylet.ObjectRef):
             return True
-        return node_type == typing.Any or isinstance(input_value, node_type)
+        return SimplePythonGraphAdapter.check_input_type(node_type, input_value)
 
     @staticmethod
     def check_node_type_equivalence(node_type: typing.Type, input_type: typing.Type) -> bool:
@@ -145,7 +145,7 @@ class RayWorkflowGraphAdapter(base.HamiltonGraphAdapter, base.ResultMixin):
         # NOTE: the type of a raylet is unknown until they are computed
         if isinstance(input_value, ray._raylet.ObjectRef):
             return True
-        return node_type == typing.Any or isinstance(input_value, node_type)
+        return SimplePythonGraphAdapter.check_input_type(node_type, input_value)
 
     @staticmethod
     def check_node_type_equivalence(node_type: typing.Type, input_type: typing.Type) -> bool:
